@@ -3,15 +3,15 @@ import itertools
 import dask
 import dask.array as da
 import numpy as np
-import xarray as xr
 import pandas as pd
+import xarray as xr
+from dask import delayed
 from owl_dev.logging import logger
 from scipy.stats import median_abs_deviation
-from dask import delayed
 from skimage.transform import rescale
 
 from ..settings import Settings
-from ..utils import trim_memory_workers, restart_workers
+from ..utils import restart_workers, trim_memory_workers
 from .detect import detect_beads
 
 
@@ -55,7 +55,7 @@ def get_stats(img, nsamples=1000):
     return z1, z2
 
 
-def beads(output_path, overwrite=False):
+def beads(output_path, overwrite=False):  # noqa: C901
     """Detect beads in a given image."""
     (output_path / "bead").mkdir(exist_ok=True)
 
@@ -77,11 +77,14 @@ def beads(output_path, overwrite=False):
             logger.debug(f"Processing section {sect}:Z{zval}")
             im = ds[sect].sel(z=zval).data.mean(axis=0)
 
-            stats = dask.compute(
-                [get_stats_nop(img) for img in im.rechunk((512, 512)).blocks]
-            )[0]
+            nim = im.rechunk((512, 512))
+            stats = []
+            for indx in itertools.product(*map(range, nim.blocks.shape)):
+                stats.append(get_stats_nop(nim.blocks[indx]))
+            stats = dask.compute(stats)[0]
             stats = sorted(stats, key=lambda x: (x[0] == 0) * 1e10 + x[1])
-            med, mad = stats[1]
+            med = np.median([s[0] for s in stats if s[0] > 0])
+            mad = np.median([s[1] for s in stats if s[0] > 0])
             logger.debug("Image background stats median %f, mad %f", med, mad)
 
             fut = []
@@ -96,7 +99,7 @@ def beads(output_path, overwrite=False):
                         min_area=Settings.beads["min_area"],
                         max_area=Settings.beads["max_area"],
                         threshold=Settings.beads["threshold"],
-                        mad=mad,
+                        stats=(med, mad),
                     )
                 )
 
